import re
import requests
from pygoogletranslation import urls

class TokenAcquirer:
    """Google Translate API token generator

    translate.google.cn uses a token to authorize the requests. If you are
    not Google, you do have this token and will have to pay for use.
    This class is the result of reverse engineering on the obfuscated and
    minified code used by Google to generate such token.

    The token is based on a seed which is updated once per hour and on the
    text that will be translated.
    Both are combined - by some strange math - in order to generate a final
    token (e.g. 464393.115905) which is used by the API to validate the
    request.

    This operation will cause an additional request to get an initial
    token from translate.google.cn.

    Example usage:
        >>> from pygoogletranslation.gauthtoken import TokenAcquirer
        >>> acquirer = TokenAcquirer()
        >>> text = 'test'
        >>> tk = acquirer.do(text)
        >>> tk
        464393.115905
    """

    def __init__(self, tkk='0', tkk_url=urls.TOKEN, proxies=None):

        if proxies is not None:
            self.proxies = proxies
        else:
            self.proxies = None

        r = requests.get(tkk_url, proxies=self.proxies)

        if r.status_code == 200:
            re_tkk = re.search("(?<=tkk=\\')[0-9.]{0,}", str(r.content.decode("utf-8")))            
            if re_tkk:
                self.tkk = re_tkk.group(0)
            else:
                self.tkk = '0'
        else:
            self.tkk = '0'


    def _xr(self, a, b):
            size_b = len(b)
            c = 0
            while c < size_b - 2:
                d = b[c + 2]
                d = ord(d[0]) - 87 if 'a' <= d else int(d)
                d = self.rshift(a, d) if '+' == b[c + 1] else a << d
                a = a + d & 4294967295 if '+' == b[c] else a ^ d

                c += 3
            return a

    def acquire(self, text):
        a = []
        # Convert text to ints
        for i in text:
            val = ord(i)
            if val < 0x10000:
                a += [val]
            else:
                # Python doesn't natively use Unicode surrogates, so account for those
                a += [
                    math.floor((val - 0x10000) / 0x400 + 0xD800),
                    math.floor((val - 0x10000) % 0x400 + 0xDC00)
                ]

        b = self.tkk
        d = b.split('.')
        b = int(d[0]) if len(d) > 1 else 0

        # assume e means char code array
        e = []
        g = 0
        size = len(a)
        while g < size:
            l = a[g]
            # just append if l is less than 128(ascii: DEL)
            if l < 128:
                e.append(l)
            # append calculated value if l is less than 2048
            else:
                if l < 2048:
                    e.append(l >> 6 | 192)
                else:
                    # append calculated value if l matches special condition
                    if (l & 64512) == 55296 and g + 1 < size and \
                            a[g + 1] & 64512 == 56320:
                        g += 1
                        l = 65536 + ((l & 1023) << 10) + (a[g] & 1023)  # This bracket is important
                        e.append(l >> 18 | 240)
                        e.append(l >> 12 & 63 | 128)
                    else:
                        e.append(l >> 12 | 224)
                    e.append(l >> 6 & 63 | 128)
                e.append(l & 63 | 128)
            g += 1
        a = b
        for i, value in enumerate(e):
            a += value
            a = self._xr(a, '+-a^+6')
        a = self._xr(a, '+-3^+b+-f')
        a ^= int(d[1]) if len(d) > 1 else 0
        if a < 0:  # pragma: nocover
            a = (a & 2147483647) + 2147483648
        a %= 1000000  # int(1E6)
        return '{}.{}'.format(a, a ^ b)

    def do(self, text):
        tk = self.acquire(text)
        return tk

    
    def rshift(self, val, n):
        """python port for '>>>'(right shift with padding)
        """
        return (val % 0x100000000) >> n

        




# -*- coding: utf-8 -*-
"""
A Translation module.

You can translate text using this module.
"""
import os
import json
import requests
import unidecode
import docx2txt
import PyPDF2
import time
from pygoogletranslation import utils, urls
from pygoogletranslation.constants import (
    LANGCODES, LANGUAGES, RPCIDS
)
from pygoogletranslation import gauthtoken
from pygoogletranslation.models import Translated, Detected


class Translator:

    def __init__(self, proxies=None, timeout=None,
                 retry=3, sleep=5, retry_messgae=False, service_url='translate.google.cn'):
        if service_url not in urls.SERVICE_URLS:
            self.service_url = 'translate.google.cn'
        else:
            self.service_url = service_url
        self.host = urls.TRANSLATE
        self.rpcids = RPCIDS
        self.transurl = 'https://{}/_/TranslateWebserverUi/data/batchexecute'.format(
            self.service_url)
        if proxies is not None:
            self.proxies = proxies
        else:
            self.proxies = None

        if timeout is not None:
            self.timeout = timeout

        self.retry = retry
        self.retry_messgae = retry_messgae
        self.sleep = sleep

    def translate(self, text, src='auto', dest='en'):
        if type(text) == list:
            i = 0
            for _text in text:
                _text = _text.replace('"', '')
                _text = _text.replace("'", "")
                _text = _text.replace("“", "")
                _text = _text.replace("”", "")
                text[i] = _text
                i += 1
        else:
            text = text.replace('"', '')
            text = text.replace("'", "")
            text = text.replace("“", "")
            text = text.replace("”", "")
        
        if src != 'auto':
            if src.lower() in LANGCODES:
                src = LANGCODES[src]
            elif src.lower() in LANGUAGES:
                src = src
            else:
                raise ValueError('invalid source language')

        if dest != 'en':
            if dest.lower() in LANGCODES:
                dest = LANGCODES[src.lower()]
            elif dest.lower() in LANGUAGES:
                dest = dest
            else:
                raise ValueError('invalid destination language')

        data = self._translate(text, src=src, dest=dest)
        return self.extract_translation(data, text)

    
    def extract_translation(self, _data, text, src='auto', dest='en'):
        if type(text) != list:
            text = [text]
        result_list = []
        c = 0
        for data in _data:
            try:
                translated = data[0][0]
            except:
                translated = ""
            extra_data = {}
            try:
                src = data[0][1]
            except Exception:  # pragma: nocover
                pass

            try:
                dest = dest
            except Exception:  # pragma: nocover
                pass

            pron = None
            try:
                pron = data[0][4]
            except Exception:  # pragma: nocover
                pass
            # put final values into a new Translated object
            result = Translated(src=src, dest=dest, origin=text[c],
                                text=translated, pronunciation=pron, extra_data=extra_data)
            result_list.append(result)
            c += 1
        if len(result_list) == 1:
            return result_list[0]
        else:
            return result_list

    def detect(self, text, **kwargs):
        """Detect language of the input text

        :param text: The source text(s) whose language you want to identify.
                     Batch detection is supported via sequence input.
        :type text: UTF-8 :class:`str`; :class:`unicode`; string sequence (list, tuple, iterator, generator)

        :rtype: Detected
        :rtype: :class:`list` (when a list is passed)

        Basic usage:
            >>> from pygoogletranslation import Translator
            >>> translator = Translator()
            >>> translator.detect('이 문장은 한글로 쓰여졌습니다.')
            <Detected lang=ko confidence=0.27041003>
            >>> translator.detect('この文章は日本語で書かれました。')
            <Detected lang=ja confidence=0.64889508>
            >>> translator.detect('This sentence is written in English.')
            <Detected lang=en confidence=0.22348526>
            >>> translator.detect('Tiu frazo estas skribita en Esperanto.')
            <Detected lang=eo confidence=0.10538048>

        Advanced usage:
            >>> langs = translator.detect(['한국어', '日本語', 'English', 'le français'])
            >>> for lang in langs:
            ...    print(lang.lang, lang.confidence)
            ko 1
            ja 0.92929292
            en 0.96954316
            fr 0.043500196
        """
        if isinstance(text, list):
            result = []
            for item in text:
                lang = self.detect(item)
                result.append(lang)
            return result

        data = self._translate(text, 'auto', 'en')

        # actual source language that will be recognized by Google Translator when the
        # src passed is equal to auto.
        src = ''
        confidence = 0.0
        try:
            src = data[0][0][2][3][5][0][0][3]
            # confidence = data[8][-2][0]
        except Exception:  # pragma: nocover
            pass
        result = Detected(lang=src, confidence=confidence)

        return result
          
    def _translate(self, text, src, dest):
        """ Generate Token for each Translation and post requset to
        google web api translation and return an response

        If the status code is 200 the request is considered as an success
        else other status code are consider as translation failure.

        """
        if type(text) != list:
            text = [text]        
        translated_list = []
        url = self.transurl
        params = utils.format_param(self.rpcids)
        for _text in text:
            trans_list = []
            tokenized_text = utils.tokenize_sentence(_text)
            for _tokenized_text in tokenized_text:
                data = utils.format_data(self.rpcids, _tokenized_text, src, dest)
                response = requests.request("POST", url, data=data, params=params, proxies=self.proxies)
                if response.status_code == 200:
                    # print(str(response.text))
                    # _format_data = utils.format_response(str(response.text))
                    trans_list.append(response.text)
                elif response.status_code == 429:
                    _format_data = self.retry_request(data, params)
                    trans_list.append(_format_data)
                else:
                    raise Exception('Unexpected status code {} from {}'.format(response.status_code, self.transurl))
                    return False
            translated_list.append(utils.format_response(trans_list))
        return translated_list

    def retry_request(self, data, params):
        """ 
        For bulk translation some times translation might failed
        beacuse of too many attempts. for such a case before hitting
        translation api wait for some time and retrying again
        """
        retry = self.retry
        sleep = self.sleep
        response = requests.request("POST", url=self.transurl, data=data, params=params, proxies=self.proxies)
        for i in range(0, retry):
            if response.status_code == 200:
                _format_data = utils.format_response(str(response.text))
                return _format_data
            elif response.status_code == 429:
                if self.retry_messgae:
                    print('retrying translation after {}s'.format(sleep))
                time.sleep(sleep)
                sleep = i * sleep
            else:
                raise Exception('Unexpected status code {} from {}'.format(
                    response.status_code, self.transurl))
                return False
        raise Exception('Unexpected status code {} from {} after retried {} loop with {}s delay'.format(
            response.status_code, self.transurl, retry, self.sleep))

    def bulktranslate(self, file, src='auto', dest='en'):
        """Translation from document (.doc, .docx, .pdf, .txt):
        ---------------------------------------------
            >>> from pygoogletranslation import Translator
            >>> translator = Translator()
            >>> translator.bulktranslate('test.txt', dest="ta")
            # <bulk translated text>
        """
        if src != 'auto':
            if src.lower() in LANGCODES:
                src = LANGCODES[src.lower()]
            elif src.lower() in LANGUAGES:
                src = src
            else:
                raise ValueError('invalid source language')

        if dest != 'en':
            if dest.lower() in LANGCODES:
                dest = LANGCODES[src.lower()]
            elif dest.lower() in LANGUAGES:
                dest = dest
            else:
                raise ValueError('invalid destination language')
        
        if not os.path.exists(file):
            raise FileNotFoundError('file {} does not exists !'.format(file))

        # Read document file, pdf file, text file
        if file.endswith('.doc') or file.endswith('.docx'):
            text = docx2txt.process(file)
        elif file.endswith('.txt'):
            _file = open(file, 'r')
            text = _file.read()
            _file.close()
        elif file.endswith('.pdf'):
            text = ''
            pdfFileObj = open(file, 'rb') 
            pdfReader = PyPDF2.PdfFileReader(pdfFileObj) 
            for i in range(0, pdfReader.numPages):
                pageObj = pdfReader.getPage(0) 
                text += pageObj.extractText()
            pdfFileObj.close() 
        else:
            raise FileNotFoundError('unsupported file format .{}.'.format(
                file.split('.'))[len(file.split('.') - 1)])
        text = text.replace('"', '')
        text = text.replace("'", "")
        text = text.replace("“", "")
        text = text.replace("”", "")
        data = self._translate(text, src=src, dest=dest)
        return self.extract_translation(data, text)
  
    def glanguage(self):
        """ Get request from google and return language and their lang codes.
        Example:
        >>> translate = Translator()
        >>> translate.glanguage()
        >>> {
                "sl": {
                    "auto": "Detect language",
                    "af": "Afrikaans",
                    "sq": "Albanian",
                },
                "tl": {
                    "af": "Afrikaans",
                    "sq": "Albanian",
                    "am": "Amharic",
                    "ar": "Arabic",
                },
                "al": {}
            }
        """
        querystring = utils.format_querystringlang()
        response = requests.get(url=self.host + 'l',
                             params=querystring, proxies=self.proxies)
        if response.status_code == 200:
            glang = json.loads(response.content)
            return glang
        else:
            raise Exception('Unexpected status code {} from {}'.format(
                response.status_code, self.host))
            return False


# -*- coding: utf-8 -*-
"""
Predefined URLs used to make google translate requests.
"""
BASE = 'https://translate.google.cn'
TOKEN = 'https://translate.google.cn/translate_a/element.js'
TRANSLATE = 'https://translate.googleapis.com/translate_a/'
TRANSLATEURL = 'https://translate.google.cn/_/TranslateWebserverUi/data/batchexecute'

SERVICE_URLS = (
    'translate.google.ac', 'translate.google.ad', 'translate.google.ae',
    'translate.google.al', 'translate.google.am', 'translate.google.as',
    'translate.google.at', 'translate.google.az', 'translate.google.ba',
    'translate.google.be', 'translate.google.bf', 'translate.google.bg',
    'translate.google.bi', 'translate.google.bj', 'translate.google.bs',
    'translate.google.bt', 'translate.google.by', 'translate.google.ca',
    'translate.google.cat', 'translate.google.cc', 'translate.google.cd',
    'translate.google.cf', 'translate.google.cg', 'translate.google.ch',
    'translate.google.ci', 'translate.google.cl', 'translate.google.cm',
    'translate.google.cn', 'translate.google.co.ao', 'translate.google.co.bw',
    'translate.google.co.ck', 'translate.google.co.cr', 'translate.google.co.id',
    'translate.google.co.il', 'translate.google.co.in', 'translate.google.co.jp',
    'translate.google.co.ke', 'translate.google.co.kr', 'translate.google.co.ls',
    'translate.google.co.ma', 'translate.google.co.mz', 'translate.google.co.nz',
    'translate.google.co.th', 'translate.google.co.tz', 'translate.google.co.ug',
    'translate.google.co.uk', 'translate.google.co.uz', 'translate.google.co.ve',
    'translate.google.co.vi', 'translate.google.co.za', 'translate.google.co.zm',
    'translate.google.co.zw', 'translate.google.cn.af', 'translate.google.cn.ag',
    'translate.google.cn.ai', 'translate.google.cn.ar', 'translate.google.cn.au',
    'translate.google.cn.bd', 'translate.google.cn.bh', 'translate.google.cn.bn',
    'translate.google.cn.bo', 'translate.google.cn.br', 'translate.google.cn.bz',
    'translate.google.cn.co', 'translate.google.cn.cu', 'translate.google.cn.cy',
    'translate.google.cn.do', 'translate.google.cn.ec', 'translate.google.cn.eg',
    'translate.google.cn.et', 'translate.google.cn.fj', 'translate.google.cn.gh',
    'translate.google.cn.gi', 'translate.google.cn.gt', 'translate.google.cn.hk',
    'translate.google.cn.jm', 'translate.google.cn.kh', 'translate.google.cn.kw',
    'translate.google.cn.lb', 'translate.google.cn.ly', 'translate.google.cn.mm',
    'translate.google.cn.mt', 'translate.google.cn.mx', 'translate.google.cn.my',
    'translate.google.cn.na', 'translate.google.cn.ng', 'translate.google.cn.ni',
    'translate.google.cn.np', 'translate.google.cn.om', 'translate.google.cn.pa',
    'translate.google.cn.pe', 'translate.google.cn.pg', 'translate.google.cn.ph',
    'translate.google.cn.pk', 'translate.google.cn.pr', 'translate.google.cn.py',
    'translate.google.cn.qa', 'translate.google.cn.sa', 'translate.google.cn.sb',
    'translate.google.cn.sg', 'translate.google.cn.sl', 'translate.google.cn.sv',
    'translate.google.cn.tj', 'translate.google.cn.tr', 'translate.google.cn.tw',
    'translate.google.cn.ua', 'translate.google.cn.uy', 'translate.google.cn.vc',
    'translate.google.cn.vn', 'translate.google.cn', 'translate.google.cv',
    'translate.google.cz', 'translate.google.de', 'translate.google.dj',
    'translate.google.dk', 'translate.google.dm', 'translate.google.dz',
    'translate.google.ee', 'translate.google.es', 'translate.google.eu',
    'translate.google.fi', 'translate.google.fm', 'translate.google.fr',
    'translate.google.ga', 'translate.google.ge', 'translate.google.gf',
    'translate.google.gg', 'translate.google.gl', 'translate.google.gm',
    'translate.google.gp', 'translate.google.gr', 'translate.google.gy',
    'translate.google.hn', 'translate.google.hr', 'translate.google.ht',
    'translate.google.hu', 'translate.google.ie', 'translate.google.im',
    'translate.google.io', 'translate.google.iq', 'translate.google.is',
    'translate.google.it', 'translate.google.je', 'translate.google.jo',
    'translate.google.kg', 'translate.google.ki', 'translate.google.kz',
    'translate.google.la', 'translate.google.li', 'translate.google.lk',
    'translate.google.lt', 'translate.google.lu', 'translate.google.lv',
    'translate.google.md', 'translate.google.me', 'translate.google.mg',
    'translate.google.mk', 'translate.google.ml', 'translate.google.mn',
    'translate.google.ms', 'translate.google.mu', 'translate.google.mv',
    'translate.google.mw', 'translate.google.ne', 'translate.google.nf',
    'translate.google.nl', 'translate.google.no', 'translate.google.nr',
    'translate.google.nu', 'translate.google.pl', 'translate.google.pn',
    'translate.google.ps', 'translate.google.pt', 'translate.google.ro',
    'translate.google.rs', 'translate.google.ru', 'translate.google.rw',
    'translate.google.sc', 'translate.google.se', 'translate.google.sh',
    'translate.google.si', 'translate.google.sk', 'translate.google.sm',
    'translate.google.sn', 'translate.google.so', 'translate.google.sr',
    'translate.google.st', 'translate.google.td', 'translate.google.tg',
    'translate.google.tk', 'translate.google.tl', 'translate.google.tm',
    'translate.google.tn', 'translate.google.to', 'translate.google.tt',
    'translate.google.us', 'translate.google.vg', 'translate.google.vu',
    'translate.google.ws'
)


"""A conversion module for googletrans"""
import json
import nltk
import unidecode
from pygoogletranslation.constants import (
    LANGCODES, LANGUAGES, RPCIDS
)
from googletrans.models import TranslatedPart

def format_querystringlang():
    querystring = {
        "client":"te",
    }
    return querystring

def format_querystring(token, text, src='auto', dest='en'):
    querystring = {
        "anno":"3",
        "format":"html",
        "key":"",
        "logld":"vTE_20201130_00",
        "client":"te",
        "v":"1.0",
        "sl": src,
        "tl": dest,
        "tk": token,
        "q": text.encode('utf-8'),
        "tc":"1",
        "sr":"1",
        "mode":"1"
    }
    return querystring


def format_param(rpcids):
    params = {
        'rpcids': rpcids,
        'bl': 'boq_translate-webserver_20201207.13_p0', 
        'soc-app': 1, 
        'soc-platform': 1, 
        'soc-device': 1, 
        'rt': 'c'
    }
    return params

def format_data(rpcids, text, src, dest):
        return {'f.req': json.dumps([[
            [
                rpcids,
                json.dumps([[text, src, dest, True],[None]], separators=(',', ':')),
                None,
                'generic',
            ],
        ]], separators=(',', ':'))}

def format_response_legacy(a):
    result = {}
    b = a.split('\n')
    li_filter = []
    flag = False
    for _b in b:
        if _b.isnumeric():
            flag = not flag
            _b = 'pygoogletranslation'
        if flag:
            li_filter.append(_b)
    fi_data = str(''.join(li_filter)).replace('"[', '[').replace(']"', ']').replace('\\n', '').replace('\\','')    
    li_data = json.loads(fi_data.split('pygoogletranslation')[1].replace('"[', '[').replace(']"', ']'))
    return li_data

def format_response(datalist):
    translatedlist = []
    for data in datalist:
        transdata = []
        token_found = False
        square_bracket_counts = [0, 0]
        resp = ''
        for line in data.split('\n'):
            token_found = token_found or f'"{RPCIDS}"' in line[:30]            
            if not token_found:
                continue

            is_in_string = False
            for index, char in enumerate(line):
                if char == '\"' and line[max(0, index - 1)] != '\\':
                    is_in_string = not is_in_string
                if not is_in_string:
                    if char == '[':
                        square_bracket_counts[0] += 1
                    elif char == ']':
                        square_bracket_counts[1] += 1

            resp += line
            if square_bracket_counts[0] == square_bracket_counts[1]:
                break

        data = json.loads(resp)
        parsed = json.loads(data[0][2])
        # not sure
        should_spacing = parsed[1][0][0][3]
        translated_parts = list(map(lambda part: TranslatedPart(part[0], part[1] if len(part) >= 2 else []), parsed[1][0][0][5]))
        translated = (' ' if should_spacing else '').join(map(lambda part: part.text, translated_parts))
        transdata.append(translated)

        
        try:
            transdata.append(parsed[2])
        except:
            transdata.append('auto')
       
        try:
            transdata.append(parsed[0][2])
        except:
            transdata.append('auto')

        # currently not available
        confidence = None

        origin_pronunciation = None
        try:
            transdata.append(parsed[0][0])
        except:
            transdata.append('')

        pronunciation = None
        try:
            transdata.append(parsed[1][0][0][1])
        except:
            transdata.append()

        translatedlist.append(transdata)
    return translatedlist

def _format_response(a):
    result = {}
    b = a.split('\n')
    li_filter = []
    flag = False
    for _b in b:
        if _b.isnumeric():
            flag = not flag
            _b = 'pygoogletranslation'
        if flag:
            # Parsing to cleanup "unescaped escaped" characters
            if '\\' in _b:
                _bp = ''
                p = 0
                while p < len(_b):
                    if _b[p:p+2] == '\\\\':
                        _bp += '\\'
                        p += 2
                    elif _b[p:p+1] == '\\':
                        if _b[p:p+2] == '\\u':
                            _bp += bytes(_b[p:p+6], 'ascii').decode('unicode-escape')
                            p += 6
                        elif _b[p:p+2] == '\\n':
                            _bp += '\n'
                            p += 2
                        else:
                            p += 1
                    else:
                        _bp += _b[p:p+1]
                        p += 1
                _b = _bp

            li_filter.append(_b)
    fi_data = str(''.join(li_filter)).replace('","[', '",[', 1).replace('\n",null', '\n,null')
    li_data = json.loads(fi_data.split('pygoogletranslation')[1], strict=False)
    return li_data
    
def tokenize_sentence(text):
    text_len = 0
    token_text = ''
    text_list = []
    if len(text) <= 5000:
        text_list.append(text)
        return text_list
    tokens = nltk.sent_tokenize(text)
    for t in tokens:
        text_len += len(t)
        if text_len < 5000:
            token_text += t
        else:
            text_list.append(token_text)
            text_len = 0
            token_text = t
    if text_len < 5000:
        text_list.append(token_text)
    return text_list


def format_translation(translated):
    text = ''
    pron = ''
    for _translated in translated:
        try:
            text += _translated[0][2][1][0][0][5][0][0]
            #text += "".join([e[0] for e in data[0][2][1][0][0][5]])
        except:
            text += fix_trans_error(_translated)
        try:
            pron += unidecode.unidecode(_translated[0][2][1][0][0][1])
        except:
            pron += ''

    for _translated in translated:
        try:
            _translated[0][2][1][0][0][5][0][0] = text
            break
        except:
            pass
    try:
        _translated[0][2][1][0][0][1] = pron
    except:
        pass
    return _translated


def fix_trans_error_legacy(translated):
    if len(translated) > 0:
        if len(translated[0]) > 2:
            if len(translated[0][2]) > 1:
                if len(translated[0][2][1]) > 0:
                    if len(translated[0][2][1][0]) > 0:
                        if len(translated[0][2][1][0][0]) > 5:
                            if len(translated[0][2][1][0][0][5]) > 0:
                                if len(translated[0][2][1][0][0][5][0]) > 0:
                                    text = translated[0][2][1][0][0][5][0][0]
                                else:
                                    text = translated[0][2][1][0][0][5][0]
                            else:
                                text = translated[0][2][1][0][0][5]
                        else:
                            text = translated[0][2][1][0][0]
                    else:
                        text = translated[0][2][1][0]
                else:
                    text = translated[0][2][1]
            else:
                text = translated[0][2]
        else:
            text = translated[0]
    else:
        text = translated
    return str(text)


def fix_trans_error(translated):
    if len(translated) > 0:
        if len(translated[0]) > 2:
            if len(translated[0][2]) > 1:
                if len(translated[0][2][1]) > 0:
                    if len(translated[0][2][1][0]) > 0:
                        if len(translated[0][2][1][0][0]) > 5:
                            if translated[0][2][1][0][0][5] is None:
                                text = translated[0][2][1][0][0][0]
                            elif len(translated[0][2][1][0][0][5]) > 0:
                                if len(translated[0][2][1][0][0][5][0]) > 0:
                                    text = translated[0][2][1][0][0][5][0][0]
                                else:
                                    text = translated[0][2][1][0][0][5][0]
                            else:
                                text = translated[0][2][1][0][0][5]
                        else:
                            text = translated[0][2][1][0][0]
                    else:
                        text = translated[0][2][1][0]
                else:
                    text = translated[0][2][1]
            else:
                text = translated[0][2]
        else:
            text = translated[0]
    else:
        text = translated
    return str(text)




"""Free Google Translate Web API for Python. Translates totally free of charge."""
__all__ = 'Translator',
__version__ = '2.0.6'


from pygoogletranslation.translate import Translator
from pygoogletranslation.constants import LANGCODES, LANGUAGES


from typing import List
class TranslatedPart:
    def __init__(self, text: str, candidates: List[str]):
        self.text = text
        self.candidates = candidates

    def __str__(self):
        return self.text

    def __dict__(self):
        return {
            'text': self.text,
            'candidates': self.candidates,
        }

class Translated:
    """Translate result object

    :param src: source langauge (default: auto)
    :param dest: destination language (default: en)
    :param origin: original text
    :param text: translated text
    :param pronunciation: pronunciation
    """
    def __init__(self, src, dest, origin, text, pronunciation, extra_data=None):
        self.src = src
        self.dest = dest
        self.origin = origin
        self.text = text
        self.pronunciation = pronunciation
        self.extra_data = extra_data

    def __str__(self):  # pragma: nocover
        return self.__unicode__()

    def __unicode__(self):  # pragma: nocover
        return (
            u'Translated(src={src}, dest={dest}, text={text}, pronunciation={pronunciation}, '
            u'original_text={origin} ,extra_data={extra_data})'.format(
                src=self.src, dest=self.dest, text=self.text,
                pronunciation=self.pronunciation, origin=self.origin,
                extra_data='"' + repr(self.extra_data)[:10] + '..."'
            )
        )


class Detected:
    """Language detection result object

    :param lang: detected language
    :param confidence: the confidence of detection result (0.00 to 1.00)
    """
    def __init__(self, lang, confidence):
        self.lang = lang
        self.confidence = confidence

    def __str__(self):  # pragma: nocover
        return self.__unicode__()

    def __unicode__(self):  # pragma: nocover
        return u'Detected(lang={lang}, confidence={confidence})'.format(
            lang=self.lang, confidence=self.confidence)



RPCIDS = 'MkEWBc'

LANGUAGES = {
    'af': 'afrikaans',
    'sq': 'albanian',
    'am': 'amharic',
    'ar': 'arabic',
    'hy': 'armenian',
    'az': 'azerbaijani',
    'eu': 'basque',
    'be': 'belarusian',
    'bn': 'bengali',
    'bs': 'bosnian',
    'bg': 'bulgarian',
    'ca': 'catalan',
    'ceb': 'cebuano',
    'ny': 'chichewa',
    'zh-cn': 'chinese (simplified)',
    'zh-tw': 'chinese (traditional)',
    'co': 'corsican',
    'hr': 'croatian',
    'cs': 'czech',
    'da': 'danish',
    'nl': 'dutch',
    'en': 'english',
    'eo': 'esperanto',
    'et': 'estonian',
    'tl': 'filipino',
    'fi': 'finnish',
    'fr': 'french',
    'fy': 'frisian',
    'gl': 'galician',
    'ka': 'georgian',
    'de': 'german',
    'el': 'greek',
    'gu': 'gujarati',
    'ht': 'haitian creole',
    'ha': 'hausa',
    'haw': 'hawaiian',
    'iw': 'hebrew',
    'he': 'hebrew',
    'hi': 'hindi',
    'hmn': 'hmong',
    'hu': 'hungarian',
    'is': 'icelandic',
    'ig': 'igbo',
    'id': 'indonesian',
    'ga': 'irish',
    'it': 'italian',
    'ja': 'japanese',
    'jw': 'javanese',
    'kn': 'kannada',
    'kk': 'kazakh',
    'km': 'khmer',
    'ko': 'korean',
    'ku': 'kurdish (kurmanji)',
    'ky': 'kyrgyz',
    'lo': 'lao',
    'la': 'latin',
    'lv': 'latvian',
    'lt': 'lithuanian',
    'lb': 'luxembourgish',
    'mk': 'macedonian',
    'mg': 'malagasy',
    'ms': 'malay',
    'ml': 'malayalam',
    'mt': 'maltese',
    'mi': 'maori',
    'mr': 'marathi',
    'mn': 'mongolian',
    'my': 'myanmar (burmese)',
    'ne': 'nepali',
    'no': 'norwegian',
    'or': 'odia',
    'ps': 'pashto',
    'fa': 'persian',
    'pl': 'polish',
    'pt': 'portuguese',
    'pa': 'punjabi',
    'ro': 'romanian',
    'ru': 'russian',
    'sm': 'samoan',
    'gd': 'scots gaelic',
    'sr': 'serbian',
    'st': 'sesotho',
    'sn': 'shona',
    'sd': 'sindhi',
    'si': 'sinhala',
    'sk': 'slovak',
    'sl': 'slovenian',
    'so': 'somali',
    'es': 'spanish',
    'su': 'sundanese',
    'sw': 'swahili',
    'sv': 'swedish',
    'tg': 'tajik',
    'ta': 'tamil',
    'te': 'telugu',
    'th': 'thai',
    'tr': 'turkish',
    'uk': 'ukrainian',
    'ur': 'urdu',
    'ug': 'uyghur',
    'uz': 'uzbek',
    'vi': 'vietnamese',
    'cy': 'welsh',
    'xh': 'xhosa',
    'yi': 'yiddish',
    'yo': 'yoruba',
    'zu': 'zulu',
}

LANGCODES = dict(map(reversed, LANGUAGES.items()))


from distutils.core import setup
from setuptools import find_packages

setup(
	name = 'pyggtranslate',
	version = '0.0.1',
	description = 'python translate by google',
	long_description = 'python translate by google', 
	author = 'pyggtranslate',
	author_email = 'lichvy@126.com',
	url = 'https://github.com/lichv/pyggtranslate',
	license = '',
	install_requires = [
		'requests>=2.25.1',
		'pymysql>=0.9.3',
		'psycopg2>=2.8.6',
		'bs4>=0.0.1',
	],
	python_requires='>=3.6',
	keywords = '',
	packages = find_packages('src'),
	package_dir = {'':'src'},
	include_package_data = True,
)
